/**
* DO NOT EDIT THIS FILE.
* See the following change record for more information,
* https://www.drupal.org/node/2815083
* @preserve
**/

(function ($, Drupal, drupalSettings, _, Backbone, JSON, storage) {
    var options = $.extend(drupalSettings.contextual, {
        strings: {
            open: Drupal.t('Open'),
            close: Drupal.t('Close')
        }
    });

    var cachedPermissionsHash = storage.getItem('Drupal.contextual.permissionsHash');
    var permissionsHash = drupalSettings.user.permissionsHash;
    if (cachedPermissionsHash !== permissionsHash) {
        if (typeof permissionsHash === 'string') {
            _.chain(storage).keys().each(function (key) {
                if (key.substring(0, 18) === 'Drupal.contextual.') {
                    storage.removeItem(key);
                }
            });
        }
        storage.setItem('Drupal.contextual.permissionsHash', permissionsHash);
    }

    function initContextual($contextual, html) {
        var $region = $contextual.closest('.contextual-region');
        if ($region.hasClass('free-contextual-region')) {
            var id = $contextual.attr('data-contextual-id');
            $region = $('body').find('[data-drupal-free-contextual-links="' + id + '"]');
        }

        var contextual = Drupal.contextual;

        $contextual.html(html).addClass('contextual').prepend(Drupal.theme('contextualTrigger'));

        var destination = 'destination=' + Drupal.encodePath(drupalSettings.path.currentPath);
        $contextual.find('.contextual-links a').each(function () {
            var url = this.getAttribute('href');
            var glue = url.indexOf('?') === -1 ? '?' : '&';
            this.setAttribute('href', url + glue + destination);
        });

        var model = new contextual.StateModel({
            title: $region.find('h2').eq(0).text().trim()
        });
        var viewOptions = $.extend({ el: $contextual, model: model }, options);
        contextual.views.push({
            visual: new contextual.VisualView(viewOptions),
            aural: new contextual.AuralView(viewOptions),
            keyboard: new contextual.KeyboardView(viewOptions)
        });
        contextual.regionViews.push(new contextual.RegionView($.extend({ el: $region, model: model }, options)));

        contextual.collection.add(model);

        $(document).trigger('drupalContextualLinkAdded', {
            $el: $contextual,
            $region: $region,
            model: model
        });

        $contextual.on('mouseenter', function () {
            var id = this.getAttribute('data-contextual-id');
            var $region = $('[data-drupal-free-contextual-links="' + id + '"].pre-close');
            $region.removeClass('pre-close');
            $region.parents('[data-drupal-free-contextual-links]').removeClass('pre-close');
        });

        $contextual.on('mouseleave', function () {
            var id = this.getAttribute('data-contextual-id');
            var $region = $('[data-drupal-free-contextual-links="' + id + '"]');
            $region.mouseleave();
        });

        $region.on('mouseenter', function () {
            setPositionVisualFreeLinks();
        });

        adjustIfNestedAndOverlapping($contextual);
    }

    function adjustIfNestedAndOverlapping($contextual) {
        var $contextuals = $contextual.parents('.contextual-region').eq(-1).find('.contextual');

        if ($contextuals.length <= 1) {
            return;
        }

        var firstTop = $contextuals.eq(0).offset().top;
        var secondTop = $contextuals.eq(1).offset().top;
        if (firstTop === secondTop) {
            var $nestedContextual = $contextuals.eq(1);

            var height = 0;
            var $trigger = $nestedContextual.find('.trigger');

            $trigger.removeClass('visually-hidden');
            height = $nestedContextual.height();
            $trigger.addClass('visually-hidden');

            $nestedContextual.css({ top: $nestedContextual.position().top + height });
        }
    }

    function setPositionVisualFreeLinks() {
        var map = [];
        $('body').find('.trigger:not(.visually-hidden)').each(function () {
            var $contextual = $(this).parent();
            var id = $contextual.attr('data-contextual-id');
            var $region = $('body').find('[data-drupal-free-contextual-links="' + id + '"]');
            if ($region.hasClass('pre-close')) {
                return true;
            }
            if ($region.length > 0) {
                var size = $(this).height() + 5;
                var top = $region.eq(0).offset().top;
                var correction = false;
                do {
                    correction = false;
                    for (var i = 0; i < map.length; i++) {
                        if (Math.abs(map[i] - top) < size) {
                            top += size;
                            correction = true;
                        }
                    }
                } while (correction);
                map.push(top);

                $contextual.parent().css({
                    top: top,
                    left: $region.eq(0).outerWidth() + $region.eq(0).offset().left
                });
            }
        });
    }

    Drupal.behaviors.contextual = {
        attach: function attach(context) {
            var $context = $(context);

            var $free_placeholders = $context.find('[data-drupal-free-contextual-links]').once('contextual-render');
            $free_placeholders.each(function () {
                var id = $(this).attr('data-drupal-free-contextual-links');

                if ($(this).find('[data-contextual-id="' + id + '"]').length === 0) {
                    if ($(this).find('[data-drupal-free-contextual-links="' + id + '"]').length === 0 && $('[data-contextual-id="' + id + '"]').length === 0) {
                        $('body').append('<div class="free-contextual-region contextual-region"><div data-contextual-id="' + id + '" class="free-contextual-link"></div></div>');
                    }
                }
            });

            var $placeholders = $context.find('[data-contextual-id]').once('contextual-render');
            if ($placeholders.length === 0) {
                return;
            }

            var ids = [];
            $placeholders.each(function () {
                ids.push($(this).attr('data-contextual-id'));
            });

            var uncachedIDs = _.filter(ids, function (contextualID) {
                var html = storage.getItem('Drupal.contextual.' + contextualID);
                if (html && html.length) {
                    window.setTimeout(function () {
                        initContextual($context.find('[data-contextual-id="' + contextualID + '"]'), html);
                    });
                    return false;
                }
                return true;
            });

            if (uncachedIDs.length > 0) {
                $.ajax({
                    url: Drupal.url('contextual/render'),
                    type: 'POST',
                    data: { 'ids[]': uncachedIDs },
                    dataType: 'json',
                    success: function success(results) {
                        _.each(results, function (html, contextualID) {
                            storage.setItem('Drupal.contextual.' + contextualID, html);

                            if (html.length > 0) {
                                $placeholders = $context.find('[data-contextual-id="' + contextualID + '"]');

                                for (var i = 0; i < $placeholders.length; i++) {
                                    initContextual($placeholders.eq(i), html);
                                }
                            }
                        });
                    }
                });
            }
        }
    };

    Drupal.contextual = {
        views: [],

        regionViews: []
    };

    Drupal.contextual.collection = new Backbone.Collection([], { model: Drupal.contextual.StateModel });

    Drupal.theme.contextualTrigger = function () {
        return '<button class="trigger visually-hidden focusable" type="button"></button>';
    };

    $(document).on('drupalContextualLinkAdded', function (event, data) {
        Drupal.ajax.bindAjaxLinks(data.$el[0]);
    });

    $(window).resize(function () {
        setPositionVisualFreeLinks();
    });
})(jQuery, Drupal, drupalSettings, _, Backbone, window.JSON, window.sessionStorage);