<?php

namespace Drupal\contextual_outside\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure youtube_gallery settings for this site.
 */
class ContextualOutsideConfig extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'contextual_outside_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'contextual_outside.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('contextual_outside.settings');

    $form['free_contextual_links_group'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Free contextual links'),
      '#open' => TRUE,
    ];
    $form['free_contextual_links_group']['free'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#default_value' => $config->get('free'),
    ];
    $form['free_contextual_links_group']['free_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Replace all contextual links'),
      '#default_value' => $config->get('free_all'),
      '#states' => [
        'visible' => [
          ':input[name="free"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->config('contextual_outside.settings')
      ->set('free', $form_state->getValue('free'))
      ->set('free_all', $form_state->getValue('free_all'))
      ->save();
    // @todo: minimize update
    drupal_flush_all_caches();
    parent::submitForm($form, $form_state);
  }

}
